@@ -24,7 +24,7 @@
 #define CHANNEL "#lunar"
 #define WEBHOOK_PORT 3000
 
-const char *lunabot_version = "0.1.3";
+const char *lunabot_version = "0.1.6";
 
 unsigned int mainloopend;
 int irc_sock;
@@ -34,6 +34,8 @@ SSL *pSSL;
 char buffer[BUFFER_SIZE];
 char buffer_log[BUFFER_SIZE * 4];
 struct MHD_Daemon *httpdaemon;
+unsigned int only_core_labels = 0; // Specific to Lunar-Linux
+unsigned int ignore_labels;
 unsigned int ignore_pending = 1;
 // IRC color codes
 #define NORMAL      "\003"   // default/restore
@@ -200,22 +202,30 @@ void *IrcConnect(void *arg) {
 	sprintf(buffer, "USER %s 0 * :GitHub PR IRC bot\r\n", NICK);
 	SSL_write(pSSL, buffer, strlen(buffer));
 
-	FILE *fp = fopen(".passwd", "r");
-	if (fp == NULL) {
-		sprintf(buffer_log, "lunabot::IrcConnect() error: Cannot open .passwd: %s", strerror(errno));
-		Log(LOCAL, buffer_log);
-		exit(1);
-	}
-	else {
-		char pass[BUFFER_SIZE - 30];
-		fgets(pass, BUFFER_SIZE - 31, fp);
-		fclose(fp);
-		if (pass[strlen(pass)-1] == '\n')
-			pass[strlen(pass)-1] = '\0';
-		sprintf(buffer, "PRIVMSG NickServ :IDENTIFY %s\r\n", pass);
+	const char *env_pass = getenv("LUNABOT_NICKSERV_PASSWORD");
+	if (env_pass != NULL && strlen(env_pass) > 0) {
+		sprintf(buffer, "PRIVMSG NickServ :IDENTIFY %s\r\n", env_pass);
 		Log(OUT, "PRIVMSG NickServ :IDENTIFY ********");
 		SSL_write(pSSL, buffer, strlen(buffer));
 	}
+	else {
+		FILE *fp = fopen(".passwd", "r");
+		if (fp == NULL) {
+			sprintf(buffer_log, "lunabot::IrcConnect() error: Cannot open .passwd: %s", strerror(errno));
+			Log(LOCAL, buffer_log);
+			exit(1);
+		}
+		else {
+			char pass[BUFFER_SIZE - 30];
+			fgets(pass, BUFFER_SIZE - 31, fp);
+			fclose(fp);
+			if (pass[strlen(pass)-1] == '\n')
+				pass[strlen(pass)-1] = '\0';
+			sprintf(buffer, "PRIVMSG NickServ :IDENTIFY %s\r\n", pass);
+			Log(OUT, "PRIVMSG NickServ :IDENTIFY ********");
+			SSL_write(pSSL, buffer, strlen(buffer));
+		}
+	}
 // Not logged in yet, exposes hostmask, needs to be sent manually in the terminal
 //	sprintf(buffer, "JOIN %s\r\n", CHANNEL);
 //	SSL_write(pSSL, buffer, strlen(buffer));
@@ -233,15 +243,14 @@ void *IrcConnect(void *arg) {
 		if (buffer[bytes-2] == '\r')
 			buffer[bytes-2] = '\0'; // Remove '\r'
 		
-		Log(IN, buffer);
 		
 		// Respond to ping requests with a pong message
 		if (strncmp(buffer, "PING", 4) == 0) {
-			sprintf(buffer_log, "PONG %s", buffer + 5);
-			Log(OUT, buffer_log);
 			sprintf(buffer2, "PONG %s\r\n", buffer + 5);
 			SSL_write(pSSL, buffer2, strlen(buffer2));
 		}
+		else
+			Log(IN, buffer);
 	}
 
 	close(irc_sock);
@@ -253,20 +262,26 @@ void *IrcConnect(void *arg) {
 int VerifySignature(const char *payload, const char *signature) {
 	unsigned int hash_len = 32;
 	unsigned char hash[hash_len];
-	char secret[1024];
+	char *secret = (char *)getenv("LUNABOT_WEBHOOK_SECRET");
 
-	FILE *fp = fopen(".secret", "r");
-	if (fp == NULL) {
-		Log(LOCAL, "lunabot::VerifySignature(): .secret file not found!");
-		exit(1);
-	}
-	else {
-		fgets(secret, 1023, fp);
-		fclose(fp);
+	unsigned int len;
+	if (secret != NULL)
+		len = strlen(secret);
+
+	if (secret == NULL || len == 0) {
+		FILE *fp = fopen(".secret", "r");
+		if (fp == NULL) {
+			Log(LOCAL, "lunabot::VerifySignature(): .secret file not found!");
+			exit(1);
+		}
+		else {
+			fgets(secret, 1023, fp);
+			fclose(fp);
 		
-		// Strip newline ending
-		if (secret[strlen(secret)-1] == '\n')
-			secret[strlen(secret)-1] = '\0';
+			// Strip newline ending
+			if (secret[strlen(secret)-1] == '\n')
+				secret[strlen(secret)-1] = '\0';
+		}
 	}
 
 	HMAC(EVP_sha256(), secret, strlen(secret), (unsigned char*)payload, strlen(payload),
@@ -303,7 +318,7 @@ void ParseJsonData(char *json_data) {
 		Log(LOCAL, buffer_log);
 		return;
 	}
-	
+
 	// Process CI build statuses
 	json_t *context = json_object_get(root, "context");
 	if (json_is_string(context)) {
@@ -367,6 +382,7 @@ void ParseJsonData(char *json_data) {
 		SendIrcMessage(message);
 		free(status_str);
 	
+		json_decref(root);
 		return;
 	}
 	
@@ -374,7 +390,81 @@ void ParseJsonData(char *json_data) {
 	json_t *action = json_object_get(root, "action");
 	json_t *pr = json_object_get(root, "pull_request");
 	if (json_is_string(action) && json_is_object(pr)) {
-		if (strcmp(json_string_value(action), "opened") == 0) {
+		if (strcmp(json_string_value(action), "labeled") == 0) {
+			if (ignore_labels) {
+				json_decref(root);
+				return;
+			}
+
+			if (only_core_labels) {
+				json_t *repo = json_object_get(root, "repository");
+				if (json_is_object(repo)) {
+					json_t *repo_name = json_object_get(repo, "name");
+					if (json_is_string(repo_name)) {
+						if (strcmp(json_string_value(repo_name), "moonbase-core") != 0) {
+							json_decref(root);
+							return;
+						}
+					}
+				}
+			}
+
+			json_t *sender = json_object_get(root, "sender");
+			if (json_is_object(sender)) {
+				json_t *username = json_object_get(sender, "login");
+				json_t *title = json_object_get(pr, "title");
+				json_t *url = json_object_get(pr, "html_url");
+				json_t *label = json_object_get(root, "label");
+				json_t *label_name = json_object_get(label, "name");
+				char message[512];
+				snprintf(message, sizeof(message), 
+						 "[%sLabels%s]: %s added the %s label to '%s' - %s",
+						 LIGHT_GREEN, NORMAL,
+						 json_string_value(username),
+						 json_string_value(label_name),
+						 json_string_value(title), 
+						 json_string_value(url));
+				SendIrcMessage(message);
+			}
+		}
+		else if (strcmp(json_string_value(action), "unlabeled") == 0) {
+			if (ignore_labels) {
+				json_decref(root);
+				return;
+			}
+
+			if (only_core_labels) {
+				json_t *repo = json_object_get(root, "repository");
+				if (json_is_object(repo)) {
+					json_t *repo_name = json_object_get(repo, "name");
+					if (json_is_string(repo_name)) {
+						if (strcmp(json_string_value(repo_name), "moonbase-core") != 0) {
+							json_decref(root);
+							return;
+						}
+					}
+				}
+			}
+
+			json_t *sender = json_object_get(root, "sender");
+			if (json_is_object(sender)) {
+				json_t *username = json_object_get(sender, "login");
+				json_t *title = json_object_get(pr, "title");
+				json_t *url = json_object_get(pr, "html_url");
+				json_t *label = json_object_get(root, "label");
+				json_t *label_name = json_object_get(label, "name");
+				char message[512];
+				snprintf(message, sizeof(message), 
+						 "[%sLabels%s]: %s removed the %s label to '%s' - %s",
+						 LIGHT_GREEN, NORMAL,
+						 json_string_value(username),
+						 json_string_value(label_name),
+						 json_string_value(title), 
+						 json_string_value(url));
+				SendIrcMessage(message);
+			}
+		}
+		else if (strcmp(json_string_value(action), "opened") == 0) {
 			json_t *title = json_object_get(pr, "title");
 			json_t *user = json_object_get(json_object_get(pr, "user"), "login");
 			json_t *url = json_object_get(pr, "html_url");
